<?php

namespace Drupal\bootstrap_ui\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures Bootstrap settings.
 */
class BootstrapSettings extends ConfigFormBase {

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * Constructs a \Drupal\bootstrap_ui\BootstrapSettings object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ThemeHandlerInterface $theme_handler) {
    parent::__construct($config_factory);

    $this->themeHandler = $theme_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('theme_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'bootstrap_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'bootstrap_ui.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // The Bootstrap UI config settings.
    $config = $this->config('bootstrap_ui.settings');

    // Handle load Bootstrap framework.
    $form['load'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Load Bootstrap framework'),
      '#default_value' => $config->get('load'),
      '#description'   => $this->t("If enabled, this module will attempt to load the Bootstrap framework for your site. To prevent loading twice, leave this option disabled if you're including the assets manually or through another module or theme."),
    ];

    // Enable right-to-left languages.
    $form['rtl'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Support RTL'),
      '#default_value' => $config->get('rtl'),
      '#description'   => $this->t('Enable right-to-left text support in Bootstrap for languages such as Persian/Arabic across your layouts, components and utilities.'),
    ];

    // Show warning missing library and lock on cdn method.
    $method = $config->get('method');
    $method_lock_change = FALSE;
    if (bootstrap_ui_check_installed() === FALSE) {
      $method = 'cdn';
      $method_lock_change = TRUE;
      $method_warning = $this->t('You cannot set local due to the Bootstrap library is missing. Please <a href=":downloadUrl" rel="external" target="_blank">Download</a> and and extract to "/libraries/bootstrap" directory.', [
        ':downloadUrl' => '//github.com/twbs/bootstrap/tags',
      ]);

      // Display warning message off.
      $form['hide'] = [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Hide warning'),
        '#default_value' => $config->get('hide') ?? FALSE,
        '#description'   => $this->t("If you want to use the CDN without installing the local library, you can turn off the warning."),
      ];
      $form['method_warning'] = [
        '#type'   => 'item',
        '#markup' => '<div class="library-status-report">' . $method_warning . '</div>',
        '#states' => [
          'visible' => [
            ':input[name="load"]' => ['checked' => TRUE],
          ],
          'invisible' => [
            ':input[name="hide"]' => ['checked' => TRUE],
          ],
        ],
      ];

      $version = !empty($config->get('version')) ? $config->get('version') : BOOTSTRAP_UI_LATEST_VERSION;
    }
    else {
      $version = ($method == 'cdn') ? $config->get('version') : bootstrap_ui_detect_version();
    }

    // Load method library from CDN or Locally.
    $form['method'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Attach method'),
      '#options'       => [
        'local'    => $this->t('Local'),
        'cdn'      => $this->t('CDN'),
      ],
      '#default_value' => $method,
      '#description'   => $this->t('These settings control how the Bootstrap library is loaded. You can choose to load from the "CDN" (External source) or from the "Local" and "Composer" (Internal library) installed.'),
      '#disabled'      => $method_lock_change,
    ];

    // Bootstrap Stable version releases.
    $form['version_wrapper'] = [
      '#type'       => 'container',
      '#attributes' => [
        'class' => [
          'bootstrap-version-wrapper',
          'container-inline',
          'form-item',
        ],
      ],
      '#states'     => [
        'disabled' => [
          'select[name="method"]' => ['value' => 'local'],
        ],
      ],
    ];

    // Get Bootstrap Stable versions.
    $versions = bootstrap_ui_version_options();
    $in_versions = in_array($version, _bootstrap_ui_array_flatten($versions));
    $form['version_wrapper']['version'] = [
      '#type'          => 'select',
      '#options'       => $versions,
      '#title'         => $this->t('Version'),
      '#default_value' => $in_versions ? $version : 'other',
    ];

    // Get Bootstrap latest stable releases.
    $releases = bootstrap_ui_version_releases();
    $in_releases = in_array($version, _bootstrap_ui_array_flatten($releases));
    $form['version_wrapper']['release'] = [
      '#type'          => 'select',
      '#options'       => $in_releases ? $releases : $releases + ['' => $this->t('Not stable')],
      '#default_value' => $in_releases ? $version : '',
      '#title'         => $this->t('Stable releases'),
      '#attributes'    => ['class' => ['bootstrap-version-release']],
      '#states'        => [
        'visible' => [
          'select[name="version"]' => ['value' => 'other'],
        ],
      ],
    ];
    $form['version_wrapper']['version_description'] = [
      '#type' => 'item',
      '#description' => $this->t('Choose the latest stable major version of Bootstrap that has been released. Select "Other" option for choose a specific release among older minor versions. Select "Auto" option to automatically detect the installed version in your local libraries directory.'),
    ];
    $form['version_wrapper']['release_notice'] = [
      '#type' => 'item',
      '#description' => $this->t('Notice: The major version structure (eg. v5.x is major) introduces significant changes, while minor versions (eg. v5.2.x is minor) focus on bug fixes. Opting for the latest minor release within each major version ensures access to improvements and bug fixes without introducing major alterations, offering stability with up-to-date enhancements.'),
      '#states'        => [
        'visible' => [
          'select[name="version"]' => ['value' => 'other'],
        ],
        'enabled' => [
          ':input[name="version"]' => ['value' => 'other'],
        ],
      ],
    ];

    // Production or minimized version.
    $form['build'] = [
      '#type'        => 'details',
      '#title'       => $this->t('Build variant'),
      '#open'        => TRUE,
      '#states'      => [
        'invisible' => [
          'select[name="compat"]' => ['checked' => TRUE],
        ],
        'disabled' => [
          ':input[name="compat"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['build']['variant'] = [
      '#type'          => 'radios',
      '#title'         => $this->t('Choose production or development library.'),
      '#options'       => [
        $this->t('Use non-minimized library (Development)'),
        $this->t('Use minimized library (Deployment)'),
      ],
      '#default_value' => $config->get('build.variant'),
      '#description'   => $this->t('These settings work with both local library and CDN methods, but this not applicable in the compat version because the compatible library is only available in minimized mode.'),
    ];

    // Vertical tabs for usability.
    $form['usability'] = [
      '#type'   => 'vertical_tabs',
      '#title'  => $this->t('Usability'),
      '#weight' => 99,
    ];

    // Get list of all installed themes.
    $themes = $this->themeHandler->listInfo();
    $active_themes = [];
    foreach ($themes as $key => $theme) {
      if ($theme->status) {
        $active_themes[$key] = $theme->info['name'];
      }
    }

    // Per-theme usability.
    $form['theme_groups'] = [
      '#type'       => 'details',
      '#title'      => $this->t('Themes'),
      '#attributes' => ['class' => ['details--settings', 'b-tooltip']],
      '#group'      => 'usability',
      '#open'       => TRUE,
    ];
    $form['theme_groups']['themes'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Installed themes'),
      '#description'   => $this->t("Specify themes by selecting their names. The list of themes to which library loading will be restricted."),
      '#options'       => $active_themes,
      '#multiple'      => TRUE,
      '#default_value' => $config->get('theme_groups.themes'),
    ];
    $form['theme_groups']['theme_negate'] = [
      '#type' => 'radios',
      '#title' => $this->t('Activate on specific themes'),
      '#title_display' => 'invisible',
      '#options' => [
        $this->t('All themes except those selected'),
        $this->t('Only the selected themes'),
      ],
      '#default_value' => $config->get('theme_groups.negate'),
    ];

    // Per-path usability.
    $form['request_path'] = [
      '#type'       => 'details',
      '#title'      => $this->t('Pages'),
      '#attributes' => ['class' => ['details--settings', 'b-tooltip']],
      '#group'      => 'usability',
    ];
    $form['request_path']['pages'] = [
      '#type'          => 'textarea',
      '#title'         => '<span class="element-invisible">' . $this->t('Pages') . '</span>',
      '#default_value' => _bootstrap_ui_array_to_string($config->get('request_path.pages')),
      '#description'   => $this->t("Specify pages by using their paths. Enter one path per line. The '*' character is a wildcard. An example path is %admin-wildcard for every user page. %front is the front page.", [
        '%admin-wildcard' => '/admin/*',
        '%front'          => '<front>',
      ]),
    ];
    $form['request_path']['page_negate'] = [
      '#type'          => 'radios',
      '#title'         => $this->t('Load Bootstrap on specific pages'),
      '#title_display' => 'invisible',
      '#options'       => [
        $this->t('All pages except those listed'),
        $this->t('Only for the listed pages'),
      ],
      '#default_value' => $config->get('request_path.negate'),
    ];

    // The Bootstrap UI settings library.
    $form['#attached']['library'][] = 'bootstrap_ui/bootstrap.settings';

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // Force detected version from installed library.
    if (bootstrap_ui_check_installed() !== FALSE && $values['method'] == 'local') {
      $version = bootstrap_ui_detect_version();
    }
    else {
      $version = $values['version'] == 'other' ? $values['release'] : $values['version'];
    }

    // Save the updated Bootstrap settings.
    $this->config('bootstrap_ui.settings')
      ->set('load', $values['load'])
      ->set('rtl', $values['rtl'])
      ->set('hide', isset($values['hide']) && $values['hide'] !== 0 ?? FALSE)
      ->set('method', $values['method'])
      ->set('version', $version)
      ->set('build.variant', $values['variant'])
      ->set('theme_groups.negate', $values['theme_negate'])
      ->set('theme_groups.themes', $values['themes'])
      ->set('request_path.negate', $values['page_negate'])
      ->set('request_path.pages', _bootstrap_ui_string_to_array($values['pages']))
      ->save();

    // Flush caches so the updated config can be checked.
    drupal_flush_all_caches();

    parent::submitForm($form, $form_state);
  }

}
